/*
 * system-dependent definitions for textutils programs. Copyright (C) 89, 90,
 * 91 Free Software Foundation, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59
 * Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Include sys/types.h before this file.  */

#include <sys/stat.h>

#ifdef STAT_MACROS_BROKEN
#undef S_ISBLK
#undef S_ISCHR
#undef S_ISDIR
#undef S_ISFIFO
#undef S_ISLNK
#undef S_ISMPB
#undef S_ISMPC
#undef S_ISNWK
#undef S_ISREG
#undef S_ISSOCK
#endif				/* STAT_MACROS_BROKEN.  */

#if !defined(S_ISBLK) && defined(S_IFBLK)
#define	S_ISBLK(m) (((m) & S_IFMT) == S_IFBLK)
#endif
#if !defined(S_ISCHR) && defined(S_IFCHR)
#define	S_ISCHR(m) (((m) & S_IFMT) == S_IFCHR)
#endif
#if !defined(S_ISDIR) && defined(S_IFDIR)
#define	S_ISDIR(m) (((m) & S_IFMT) == S_IFDIR)
#endif
#if !defined(S_ISREG) && defined(S_IFREG)
#define	S_ISREG(m) (((m) & S_IFMT) == S_IFREG)
#endif
#if !defined(S_ISFIFO) && defined(S_IFIFO)
#define	S_ISFIFO(m) (((m) & S_IFMT) == S_IFIFO)
#endif
#if !defined(S_ISLNK) && defined(S_IFLNK)
#define	S_ISLNK(m) (((m) & S_IFMT) == S_IFLNK)
#endif
#if !defined(S_ISSOCK) && defined(S_IFSOCK)
#define	S_ISSOCK(m) (((m) & S_IFMT) == S_IFSOCK)
#endif
#if !defined(S_ISMPB) && defined(S_IFMPB)	/* V7 */
#define S_ISMPB(m) (((m) & S_IFMT) == S_IFMPB)
#define S_ISMPC(m) (((m) & S_IFMT) == S_IFMPC)
#endif
#if !defined(S_ISNWK) && defined(S_IFNWK)	/* HP/UX */
#define S_ISNWK(m) (((m) & S_IFMT) == S_IFNWK)
#endif
#if !defined(HAVE_MKFIFO)
#define mkfifo(path, mode) (mknod ((path), (mode) | S_IFIFO, 0))
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifndef _POSIX_VERSION
off_t           lseek();
#endif

#ifndef STDIN_FILENO
#define STDIN_FILENO 0
#endif

#ifndef STDOUT_FILENO
#define STDOUT_FILENO 1
#endif

#ifndef STDERR_FILENO
#define STDERR_FILENO 2
#endif

/*
 * Don't use bcopy!  Use memmove if source and destination may overlap,
 * memcpy otherwise.
 */

#ifdef HAVE_STRING_H
#if !STDC_HEADERS && HAVE_MEMORY_H
#include <memory.h>
#endif
#include <string.h>
#else
#include <strings.h>
char           *memchr();
#endif

#include <errno.h>
#ifndef errno
extern int      errno;
#endif

#ifdef STDC_HEADERS
#include <stdlib.h>
#else
char           *getenv();
#endif

/*
 * The following test is to work around the gross typo in systems like Sony
 * NEWS-OS Release 4.0C, whereby EXIT_FAILURE is defined to 0, not 1.
 */
#if !EXIT_FAILURE
#undef EXIT_FAILURE
#define EXIT_FAILURE 1
#endif

#ifndef EXIT_SUCCESS
#define EXIT_SUCCESS 0
#endif

#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#else
#include <sys/file.h>
#endif

#if !defined(SEEK_SET)
#define SEEK_SET 0
#define SEEK_CUR 1
#define SEEK_END 2
#endif

#ifndef _POSIX_SOURCE
#include <sys/param.h>
#endif

/*
 * Get or fake the disk device blocksize. Usually defined by sys/param.h (if
 * at all).
 */
#if !defined(DEV_BSIZE) && defined(BSIZE)
#define DEV_BSIZE BSIZE
#endif
#if !defined(DEV_BSIZE) && defined(BBSIZE)	/* SGI */
#define DEV_BSIZE BBSIZE
#endif
#ifndef DEV_BSIZE
#define DEV_BSIZE 4096
#endif

/*
 * Extract or fake data from a `struct stat'. ST_BLKSIZE: Optimal I/O
 * blocksize for the file, in bytes.
 */
#ifndef HAVE_ST_BLKSIZE
#define ST_BLKSIZE(statbuf) DEV_BSIZE
#else				/* HAVE_ST_BLKSIZE */
/* Some systems, like Sequents, return st_blksize of 0 on pipes. */
#define ST_BLKSIZE(statbuf) ((statbuf).st_blksize > 0 \
			       ? (statbuf).st_blksize : DEV_BSIZE)
#endif				/* HAVE_ST_BLKSIZE */

#ifndef S_ISLNK
#define lstat stat
#endif

#ifndef RETSIGTYPE
#define RETSIGTYPE void
#endif

#include <ctype.h>

/*
 * Jim Meyering writes:
 * 
 * "... Some ctype macros are valid only for character codes that isascii says
 * are ASCII (SGI's IRIX-4.0.5 is one such system --when using /bin/cc or gcc
 * but without giving an ansi option).  So, all ctype uses should be through
 * macros like ISPRINT...  If STDC_HEADERS is defined, then autoconf has
 * verified that the ctype macros don't need to be guarded with references to
 * isascii. ... Defining isascii to 1 should let any compiler worth its salt
 * eliminate the && through constant folding."
 * 
 * Bruno Haible adds:
 * 
 * "... Furthermore, isupper(c) etc. have an undefined result if c is outside
 * the range -1 <= c <= 255. One is tempted to write isupper(c) with c being
 * of type `char', but this is wrong if c is an 8-bit character >= 128 which
 * gets sign-extended to a negative value. The macro ISUPPER protects against
 * this as well."
 */

#if defined (STDC_HEADERS) || (!defined (isascii) && !defined (HAVE_ISASCII))
#define IN_CTYPE_DOMAIN(c) 1
#else
#define IN_CTYPE_DOMAIN(c) isascii(c)
#endif

#ifdef isblank
#define ISBLANK(c) (IN_CTYPE_DOMAIN (c) && isblank (c))
#else
#define ISBLANK(c) ((c) == ' ' || (c) == '\t')
#endif
#ifdef isgraph
#define ISGRAPH(c) (IN_CTYPE_DOMAIN (c) && isgraph (c))
#else
#define ISGRAPH(c) (IN_CTYPE_DOMAIN (c) && isprint (c) && !isspace (c))
#endif

#define ISPRINT(c) (IN_CTYPE_DOMAIN (c) && isprint (c))
#define ISALNUM(c) (IN_CTYPE_DOMAIN (c) && isalnum (c))
#define ISALPHA(c) (IN_CTYPE_DOMAIN (c) && isalpha (c))
#define ISCNTRL(c) (IN_CTYPE_DOMAIN (c) && iscntrl (c))
#define ISLOWER(c) (IN_CTYPE_DOMAIN (c) && islower (c))
#define ISPUNCT(c) (IN_CTYPE_DOMAIN (c) && ispunct (c))
#define ISSPACE(c) (IN_CTYPE_DOMAIN (c) && isspace (c))
#define ISUPPER(c) (IN_CTYPE_DOMAIN (c) && isupper (c))
#define ISXDIGIT(c) (IN_CTYPE_DOMAIN (c) && isxdigit (c))
#define ISDIGIT_LOCALE(c) (IN_CTYPE_DOMAIN (c) && isdigit (c))

/*
 * ISDIGIT differs from ISDIGIT_LOCALE, as follows: - Its arg may be any int
 * or unsigned int; it need not be an unsigned char. - It's guaranteed to
 * evaluate its argument exactly once. - It's typically faster. Posix
 * 1003.2-1992 section 2.5.2.1 page 50 lines 1556-1558 says that only '0'
 * through '9' are digits.  Prefer ISDIGIT to ISDIGIT_LOCALE unless it's
 * important to use the locale's definition of `digit' even when the host
 * does not conform to Posix.
 */
#define ISDIGIT(c) ((unsigned) (c) - '0' <= 9)

#ifndef __P
#if PROTOTYPES
#define __P(Args) Args
#else
#define __P(Args) ()
#endif
#endif

/* Take care of NLS matters.  */

#if HAVE_LOCALE_H
#include <locale.h>
#endif
#if !HAVE_SETLOCALE
#define setlocale(Category, Locale)	/* empty */
#endif

#if ENABLE_NLS
#include <libintl.h>
#define _(Text) gettext (Text)
#else
#undef bindtextdomain
#define bindtextdomain(Domain, Directory)	/* empty */
#undef textdomain
#define textdomain(Domain)	/* empty */
#define _(Text) Text
#endif
